Adbc.StreamResult (adbc v0.9.0)

Copy Markdown View Source

Represents an unmaterialized Arrow stream from a query.

This struct can only be used within the callback passed to Adbc.Connection.query_pointer/4. The stream can only be consumed once - after being passed to bulk_insert/3 or other operations, it becomes invalid.

It contains:

  • :ref - internal reference to the stream (do not use directly)
  • :conn - internal connection pid (do not use directly)
  • :pointer - pointer to the ArrowArrayStream (integer memory address)
  • :num_rows - the number of rows affected by the query, may be nil for queries depending on the database driver

Summary

Types

t()

@type t() :: %Adbc.StreamResult{
  conn: pid(),
  num_rows: non_neg_integer() | nil,
  pointer: non_neg_integer(),
  ref: reference()
}